

class DataField:
    """
    Class to retrieve data by graph traversal. 
    Can currently only get data from given item, and items with an edge to given item.

    Example usage for items labeled by the Doccano plugin:
    ```
    items = client.search({"type": "Message"})

    data_fields = [
        DataField("content"),
        DataField("label", edge="label", item_property="index"),
        DataField("label_str", edge="label", item_property="name")
    ]
    data_dict = {
        data_field.name: data_field.get_data(items) for data_field in data_fields
    }
    ```
    """
    def __init__(self, name, item_property=None, edge=None):
        """
        TODO multiple edges (for example, EmailMessage.account.owner.name)
        TODO 
        """
        self.name = name
        self.item_property = item_property if item_property is not None else name
        self.edge = edge

        
    def get_data(self, items):
        # No edges, data is a property of item
        if self.edge is None:
            return [getattr(item, self.item_property, None) for item in items]
           
        # data is property of item.edge[-1]
        data = []
        for item in items:
            targets = getattr(item, self.edge)
            if len(targets):
                target = targets[-1]
                data.append(getattr(target, self.item_property, None))
            else:
                data.append(None)
        return data


    @classmethod
    def from_str(cls, s, name=None):
        """Generates DataField from str. Example:
        
        DataField.from_str("Content", "content") == DataField(
            name="Content",
            item_property="content")
        DataField.from_str("Label", "label.index") == DataField(
            name="Label",
            edge="label",
            item_property="index")
        """

        if name == None:
            name = s
        s = s.split(".")
        if len(s) == 1:
            return cls(name, item_property=s[0])
        elif len(s) == 2:
            return cls(
                name=name,
                edge=s[0],
                item_property=s[1]
            )
        else:
            raise NotImplemented(f"Could not parse DataField {s}")
